<?php

    namespace FederationCLI;

    class Utilities
    {
        /**
         * Parses the shell input into a command and arguments array
         *
         * @param string $input
         * @return array
         */
        public static function parseShellInput(string $input): array
        {
            $parsed = explode(' ', $input);
            $command = array_shift($parsed);
            $args = $parsed;

            return [
                'command' => $command,
                'args' => $args
            ];
        }

        /**
         * Parses a boolean value from a string
         *
         * @param $input
         * @return bool
         */
        public static function parseBoolean($input): bool
        {
            if(is_null($input))
                return false;

            if(is_bool($input))
                return $input;

            if(is_numeric($input))
                return (bool) $input;

            if(is_string($input))
                $input = trim($input);

            switch(strtolower($input))
            {
                case 'true':
                case 'yes':
                case 'y':
                case '1':
                    return true;

                default:
                case 'false':
                case 'no':
                case 'n':
                case '0':
                    return false;
            }
        }

        /**
         * Prompts the user for an input value
         *
         * @param string|null $prompt
         * @param string|null $default_value
         * @return string|null
         */
        public static function promptInput(?string $prompt=null, ?string $default_value=null): ?string
        {
            if($prompt)
                print($prompt . ' ');

            $input = trim(fgets(STDIN));

            if(!$input && $default_value)
                $input = $default_value;

            return $input;
        }

        /**
         * Prompts the user for a boolean value
         *
         * @param string|null $prompt
         * @param bool $default_value
         * @return bool
         */
        public static function promptYesNo(?string $prompt=null, bool $default_value=false): bool
        {
            if($prompt)
                print($prompt . ' ');

            $input = trim(fgets(STDIN));

            if(!$input && $default_value)
                $input = $default_value;

            return self::parseBoolean($input);
        }

        /**
         * Prints data in a formatted manner
         *
         * @param $banner_text
         * @param $data
         * @param int $body_width
         * @return void
         */
        public static function printData($banner_text, $data, int $body_width = 70)
        {
            // Padding and wrap for the longest key
            $max_key_len = max(array_map('strlen', array_keys($data)));

            // Adjust padding for body_width
            $padding = $body_width - ($max_key_len + 4); // Additional 2 spaces for initial padding

            // Banner
            $banner_width = $body_width + 2;
            echo str_repeat("*", $banner_width) . PHP_EOL;
            echo "* " . str_pad($banner_text, $banner_width - 4, ' ', STR_PAD_RIGHT) . " *" . PHP_EOL;
            echo str_repeat("*", $banner_width) . PHP_EOL;

            // Print data
            foreach ($data as $key => $value) {
                // Split value into lines if it's too long
                $lines = str_split($value, $padding);

                // Print lines
                foreach ($lines as $i => $line) {
                    if ($i == 0) {
                        // First line - print key and value
                        echo '  ' . str_pad(strtoupper($key), $max_key_len, ' ', STR_PAD_RIGHT) . '  ' . $line . PHP_EOL;
                    } else {
                        // Additional lines - only value
                        echo str_repeat(' ', $max_key_len + 4) . $line . PHP_EOL;
                    }
                }
            }
        }

    }