<?php

    /** @noinspection PhpMissingFieldTypeInspection */

    namespace FederationLib\Classes;

    use Exception;
    use RuntimeException;

    class Security
    {
        /**
         * The number of digits in the generated code.
         *
         * @var int
         */
        private static $digits = 6;

        /**
         * The number of seconds a code is valid.
         *
         * @var int
         */
        private static $time_step = 30;

        /**
         * The number of seconds the clock is allowed to drift.
         *
         * @var int
         */
        private static $time_offset = 0;

        /**
         * The hash algorithm used to generate the code.
         *
         * @var string
         */
        private static $hash_algorithm = 'sha1';

        /**
         * Generates a secret TOTP key.
         *
         * @param $length
         * @return string
         */
        public static function generateSecret($length = 20): string
        {
            try
            {
                return Base32::encode(random_bytes($length));
            }
            catch(Exception $e)
            {
                throw new RuntimeException('Failed to generate secret: ' . $e->getMessage(), $e);
            }
        }

        /**
         * Generates a TOTP code based on a secret and timestamp.
         *
         * @param $secret
         * @param $timestamp
         * @return string
         */
        public static function generateCode($secret, $timestamp=null): string
        {
            if (!$timestamp)
            {
                $timestamp = time();
            }

            $hash = hash_hmac(self::$hash_algorithm, pack('N*', 0, $timestamp / self::$time_step + self::$time_offset), Base32::decode($secret), true);
            $offset = ord(substr($hash, -1)) & 0x0F;
            $value = unpack('N', substr($hash, $offset, 4))[1] & 0x7FFFFFFF;
            $modulo = 10 ** self::$digits;
            return str_pad($value % $modulo, self::$digits, '0', STR_PAD_LEFT);
        }

        /**
         * Validates a TOTP code.
         *
         * @param $secret
         * @param $code
         * @param $window
         * @return bool
         */
        public static function validateCode($secret, $code, $window = 1): bool
        {
            $timestamp = time();

            for ($i = -$window; $i <= $window; $i++)
            {
                $generatedCode = self::generateCode($secret, $timestamp + $i * self::$time_step);
                if ($code === $generatedCode)
                {
                    return true;
                }

            }
            return false;
        }

        /**
         * Same as gettype() but returns only what the user should see.
         *
         * @param mixed $value
         * @return string
         */
        public static function gettype(mixed $value): string
        {
            return match (strtolower(gettype($value)))
            {
                'boolean' => 'boolean',
                'integer' => 'integer',
                'double' => 'float',
                'string' => 'string',
                'array' => 'array',
                default => 'null',
            };
        }
    }