<?php

    namespace FederationLib\Managers\MetadataManagers;

    use Doctrine\DBAL\ParameterType;
    use Exception;
    use FederationLib\Classes\Configuration;
    use FederationLib\Classes\Database;
    use FederationLib\Classes\Utilities;
    use FederationLib\Enums\DatabaseTables;
    use FederationLib\Enums\Misc;
    use FederationLib\Exceptions\DatabaseException;
    use FederationLib\Exceptions\Standard\InvalidPeerMetadataException;
    use FederationLib\Exceptions\Standard\PeerMetadataNotFoundException;
    use FederationLib\Interfaces\PeerMetadataInterface;
    use FederationLib\Interfaces\PeerMetadataManagerInterface;
    use FederationLib\Managers\RedisConnectionManager;
    use FederationLib\Objects\ClientRecord;
    use FederationLib\Objects\ParsedFederatedAddress;
    use FederationLib\Objects\Standard\PeerMetadata\TelegramUserMetadata;
    use LogLib\Log;

    class TelegramUserManager implements PeerMetadataManagerInterface
    {
        /**
         * @param ClientRecord|string $client_uuid
         * @param TelegramUserMetadata $peer_metadata
         * @return string
         * @throws DatabaseException
         * @throws InvalidPeerMetadataException
         */
        public function syncMetadata(ClientRecord|string $client_uuid, PeerMetadataInterface $peer_metadata): string
        {
            if($client_uuid instanceof ClientRecord)
            {
                $client_uuid = $client_uuid->getUuid();
            }

            $peer_metadata->validate();
            $cache_key = sprintf('telegram_user_metadata:%s', $peer_metadata->getFederatedAddress());
            $redis_client = null;

            if(Configuration::getObjectCacheEnabled('peer_objects'))
            {
                try
                {
                    $redis_client = RedisConnectionManager::getConnectionFromConfig('peer_objects');

                    if($redis_client->exists($cache_key))
                    {
                        $redis_client->del($cache_key);
                    }
                }
                catch(Exception $e)
                {
                    Log::warning(Misc::FEDERATIONLIB, sprintf('There was an error with the cache system while tyring to sync peer metadata with the Redis server: %s', $e->getMessage()), $e);
                }
            }

            try
            {
                $old_metadata = $this->getMetadata($peer_metadata->getFederatedAddress());
            }
            catch(PeerMetadataNotFoundException $e)
            {
                unset($e);

                // If it doesn't exist, we create it instead of updating it
                return $this->registerMetadata($client_uuid, $peer_metadata);
            }

            // Take the old metadata and get the differences
            $differences = Utilities::metadataDifferences($old_metadata, $peer_metadata);

            try
            {
                // If there are no differences, then we don't need to update the database
                if(count($differences) === 0)
                {
                    if($redis_client !== null && Configuration::getObjectCacheTtl('peer_objects') > 0)
                    {
                        // Reset the TTL since we just accessed it
                        $redis_client->expire($cache_key, Configuration::getObjectCacheTtl('peer_objects'));
                    }

                    return $peer_metadata->getFederatedAddress();
                }

                $qb = Database::getConnection()->createQueryBuilder();
                $qb->update(DatabaseTables::PEERS_TELEGRAM_USER);
                $qb->where('federated_address = :federated_address');
                $qb->setParameter('federated_address', $peer_metadata->getFederatedAddress());
                $qb->set('updated_timestamp', $qb->createNamedParameter(time(), ParameterType::INTEGER));
                $qb->set('updated_client', $qb->createNamedParameter($client_uuid));
                $qb->setMaxResults(1);

                foreach($differences as $key => $value)
                {
                    $qb->set($key, $qb->createNamedParameter($value));
                }

                $qb->executeQuery();
            }
            catch(Exception $e)
            {
                throw new DatabaseException(sprintf('There was a database error while trying to sync peer metadata: %s', $e->getMessage()), $e);
            }

            if($redis_client !== null && count($differences) > 0)
            {
                try
                {
                    foreach($differences as $key => $value)
                    {
                        $redis_client->hSet($cache_key, $key, $value);
                    }
                }
                catch(Exception $e)
                {
                    Log::warning(Misc::FEDERATIONLIB, sprintf('There was an error with the cache system while tyring to sync peer metadata with the Redis server: %s', $e->getMessage()), $e);
                }
            }

            return $peer_metadata->getFederatedAddress();
        }

        /**
         * Registers the client's metadata with the database
         *
         * @param ClientRecord|string $client_uuid
         * @param TelegramUserMetadata $peer_metadata
         * @return string
         * @throws DatabaseException
         */
        public function registerMetadata(ClientRecord|string $client_uuid, PeerMetadataInterface $peer_metadata): string
        {
            if($client_uuid instanceof ClientRecord)
            {
                $client_uuid = $client_uuid->getUuid();
            }

            try
            {
                $peer_metadata->validate();
                $qb = Database::getConnection()->createQueryBuilder();
                $qb->insert(DatabaseTables::PEERS_TELEGRAM_USER);

                $qb->setValue('federated_address', $qb->createNamedParameter($peer_metadata->getFederatedAddress()));
                $qb->setValue('updated_timestamp', $qb->createNamedParameter(time(), ParameterType::INTEGER));
                $qb->setValue('updated_client', $qb->createNamedParameter($client_uuid));

                foreach($peer_metadata->toArray() as $key => $value)
                {
                    switch(gettype($value))
                    {
                        case 'array':
                            $qb->setValue($key, $qb->createNamedParameter(implode(',', $value)));
                            break;
                        case 'NULL':
                            $qb->setValue($key, $qb->createNamedParameter(null, ParameterType::NULL));
                            break;
                        case 'boolean':
                            $qb->setValue($key, $qb->createNamedParameter((int)$value, ParameterType::INTEGER));
                            break;
                        case 'integer':
                            $qb->setValue($key, $qb->createNamedParameter($value, ParameterType::INTEGER));
                            break;

                        default:
                            $qb->setValue($key, $qb->createNamedParameter($value));
                            break;
                    }
                }

                $qb->executeQuery();
            }
            catch(Exception $e)
            {
                throw new DatabaseException(sprintf('There was a database error while trying to register peer metadata: %s', $e->getMessage()), $e);
            }

            return $peer_metadata->getFederatedAddress();
        }

        /**
         * @param ParsedFederatedAddress|string $federated_address
         * @return TelegramUserMetadata
         * @throws DatabaseException
         * @throws PeerMetadataNotFoundException
         */
        public function getMetadata(ParsedFederatedAddress|string $federated_address): TelegramUserMetadata
        {
            if(Configuration::getObjectCacheEnabled('peer_objects'))
            {
                try
                {
                    $redis = RedisConnectionManager::getConnectionFromConfig('peer_objects');
                    $key = sprintf('telegram_user_metadata:%s', $federated_address->getAddress());

                    if($redis->exists($key))
                    {
                        $telegram_user_metadata = TelegramUserMetadata::fromArray($redis->hGetAll($key), true);

                        if(Configuration::getObjectCacheTTL('peer_objects') > 0)
                        {
                            $redis->expire($key, Configuration::getObjectCacheTTL('peer_objects'));
                        }

                        Log::debug(Misc::FEDERATIONLIB, sprintf('Loaded peer metadata object %s from cache', $federated_address->getAddress()));
                        return $telegram_user_metadata;
                    }
                }
                catch(Exception $e)
                {
                    Log::warning(Misc::FEDERATIONLIB, sprintf('Failed to load peer metadata object %s from cache: %s', $federated_address->getAddress(), $e->getMessage()));
                }
            }

            $qb = Database::getConnection()->createQueryBuilder();
            $qb->select(
                'id',
                'type',
                'title',
                'username',
                'first_name',
                'last_name',
                'is_forum',
                'updated_timestamp',
                'updated_client'
            );
            $qb->from(DatabaseTables::PEERS_TELEGRAM_USER);
            $qb->where('federated_address = :federated_address');
            $qb->setParameter('federated_address', $federated_address->getAddress());
            $qb->setMaxResults(1);

            try
            {
                $result = $qb->executeQuery();

                if($result->rowCount() === 0)
                {
                    throw new PeerMetadataNotFoundException(sprintf('PeerRecord metadata not found for federated address %s', $federated_address->getAddress()));
                }
                
                $telegram_user_metadata = TelegramUserMetadata::fromArray($result->fetchAssociative(), true);
            }
            catch(PeerMetadataNotFoundException $e)
            {
                throw $e;
            }
            catch(Exception $e)
            {
                throw new DatabaseException(sprintf('Failed to get peer metadata for federated address %s', $federated_address->getAddress()), $e);
            }
            
            if(Configuration::isCacheSystemEnabled() && Configuration::getObjectCacheEnabled('peer_objects'))
            {
                try
                {
                    $redis = RedisConnectionManager::getConnectionFromConfig('peer_objects');

                    $key = sprintf('telegram_user_metadata:%s', $federated_address);
                    $redis->hMSet($key, $telegram_user_metadata->toArray());
                    
                    if(Configuration::getObjectCacheTTL('peer_objects') > 0)
                    {
                        $redis->expire($key, Configuration::getObjectCacheTTL('peer_objects'));
                    }
                    
                    Log::debug(Misc::FEDERATIONLIB, sprintf('Cached peer metadata object %s', $federated_address->getAddress()));
                }
                catch(Exception $e)
                {
                    Log::warning(Misc::FEDERATIONLIB, sprintf('Failed to cache peer metadata object %s: %s', $federated_address->getAddress(), $e->getMessage()));
                }
            }
            
            return $telegram_user_metadata;
        }
    }