<?php

    /** @noinspection PhpMissingFieldTypeInspection */

    namespace FederationLib\Objects;

    use InvalidArgumentException;

    class ParsedFederatedAddress
    {
        /**
         * @var string
         */
        private $source;

        /**
         * @var string
         */
        private $peer_type;

        /**
         * @var string
         */
        private $unique_identifier;

        /**
         * Public Constructor, parses the federated address and sets the properties
         *
         * @param string $federated_address
         */
        public function __construct(string $federated_address)
        {
            preg_match("/(?<source>[a-z0-9]+)\.(?<type>[a-z0-9]+):(?<id>.+)/", $federated_address, $matches);

            // Validate the federated address
            if (empty($matches))
            {
                throw new InvalidArgumentException('Invalid Federated Address');
            }

            $this->source = $matches['source'];
            $this->peer_type = $matches['type'];
            $this->unique_identifier = $matches['id'];
        }

        /**
         * Returns the platform source of the peer
         *
         * @return string
         */
        public function getSource(): string
        {
            return strtolower($this->source);
        }

        /**
         * Returns the peer type of the platform
         *
         * @return string
         */
        public function getPeerType(): string
        {
            return strtolower($this->peer_type);
        }

        /**
         * Returns the full peer class name without the ID
         * Eg; telegram.user
         *
         * @return string
         */
        public function getPeerClass(): string
        {
            return strtolower(sprintf('%s.%s', $this->source, $this->peer_type));
        }

        /**
         * Returns the Unique Identifier of the peer
         *
         * @return string
         */
        public function getUniqueIdentifier(): string
        {
            return $this->unique_identifier;
        }

        /**
         * Returns the Standard Federated Address of the peer
         *
         * @return string
         */
        public function getAddress(): string
        {
            return sprintf('%s.%s:%s', $this->source, $this->peer_type, $this->unique_identifier);
        }

        /**
         * Returns the Standard Federated Address of the peer
         *
         * @return string
         */
        public function __toString(): string
        {
            return $this->getAddress();
        }

    }