<?php

    /** @noinspection PhpMissingFieldTypeInspection */

    namespace FederationLib\Objects\Standard\PeerMetadata;

    use FederationLib\Classes\Validate;
    use FederationLib\Enums\Standard\InternetPeerType;
    use FederationLib\Exceptions\Standard\InvalidPeerMetadataException;
    use FederationLib\Interfaces\PeerMetadataInterface;
    use FederationLib\Objects\ParsedFederatedAddress;

    class TelegramChatMetadata implements PeerMetadataInterface
    {
        /**
         * @var int
         */
        private $id;

        /**
         * @var string
         */
        private $type;

        /**
         * @var string|null
         */
        private $title;

        /**
         * @var string|null
         */
        private $username;

        /**
         * @var string|null
         */
        private $first_name;

        /**
         * @var string|null
         */
        private $last_name;

        /**
         * @var bool
         */
        private $is_forum;

        /**
         * @var int|null
         */
        private $updated_timestamp;

        /**
         * @var string|null
         */
        private $updated_client;

        /**
         * Validates the metadata, throws an exception if the metadata is invalid.
         *
         * @return void
         * @throws InvalidPeerMetadataException
         */
        public function validate(): void
        {
            $required_properties = [
                'id' => 'integer',
                'type' => 'string'
            ];

            $optional_properties = [
                'title' => 'string',
                'username' => 'string',
                'first_name' => 'string',
                'last_name' => 'string',
                'is_forum' => 'boolean'
            ];

            Validate::validateMetadata($this->toArray(), $required_properties, $optional_properties);

            if(!in_array(strtolower($this->type), ['private', 'group', 'supergroup', 'channel']))
            {
                throw new InvalidPeerMetadataException(sprintf('The metadata contains an invalid value for the property "type": got %s"', $this->type));
            }
        }

        /**
         * Indicates whether this peer is automated
         * (in this case it's never because it's an internet peer)
         *
         * @return bool
         */
        public function isAutomated(): bool
        {
            return false;
        }

        /**
         * Returns the platform-specific ID for this peer
         *
         * @return string
         */
        public function getPlatformId(): string
        {
            return $this->id;
        }

        /**
         * Returns the full federated address for this peer
         *
         * @return ParsedFederatedAddress
         */
        public function getFederatedAddress(): ParsedFederatedAddress
        {
            return new ParsedFederatedAddress(sprintf('%s:%s', InternetPeerType::TELEGRAM_CHAT, $this->id));
        }

        /**
         * Returns the Client UUID that last updated this metadata
         *
         * @return string|null
         */
        public function getUpdatedClient(): ?string
        {
            return $this->updated_client;
        }

        /**
         * Sets the client UUID that last updated this metadata
         *
         * @param string|null $client_uuid
         * @return void
         */
        public function setUpdatedClient(?string $client_uuid): void
        {
            $this->updated_client = $client_uuid;
        }

        /**
         * Returns the Unix Timestamp for when this metadata was last updated
         *
         * @return int|null
         */
        public function getLastUpdatedTimestamp(): ?int
        {
            return $this->updated_timestamp;
        }

        /**
         * Sets the last updated timestamp
         *
         * @param int|null $timestamp
         * @return void
         */
        public function setLastUpdatedTimestamp(?int $timestamp): void
        {
            $this->updated_timestamp = $timestamp;
        }

        /**
         * Unique identifier for this chat. This number may have more than 32 significant bits and some programming
         * languages may have difficulty/silent defects in interpreting it. But it has at most 52 significant bits, so
         * a signed 64-bit integer or double-precision float type are safe for storing this identifier.
         *
         * @return int
         */
        public function getId(): int
        {
            return $this->id;
        }

        /**
         * Type of chat, can be either “private”, “group”, “supergroup” or “channel”
         *
         * @return string
         */
        public function getType(): string
        {
            return $this->type;
        }

        /**
         * Optional. Title, for supergroups, channels and group chats
         *
         * @return string|null
         */
        public function getTitle(): ?string
        {
            return $this->title;
        }

        /**
         * Optional. Username, for private chats, supergroups and channels if available
         *
         * @return string|null
         */
        public function getUsername(): ?string
        {
            return $this->username;
        }

        /**
         *  Optional. First name of the other party in a private chat
         *
         * @return string|null
         */
        public function getFirstName(): ?string
        {
            return $this->first_name;
        }

        /**
         * Optional. Last name of the other party in a private chat
         *
         * @return string|null
         */
        public function getLastName(): ?string
        {
            return $this->last_name;
        }

        /**
         * Optional. True, if the supergroup chat is a forum (has topics enabled)
         *
         * @link https://telegram.org/blog/topics-in-groups-collectible-usernames#topics-in-groups
         * @return bool
         */
        public function isForum(): bool
        {
            return $this->is_forum;
        }

        /**
         * Returns an array representation of this object
         *
         * @param bool $raw
         * @return array
         */
        public function toArray(bool $raw=false): array
        {
            $return = [
                'id' => $this->id,
                'type' => $this->type,
                'title' => $this->title,
                'username' => $this->username,
                'first_name' => $this->first_name,
                'last_name' => $this->last_name,
                'is_forum' => $this->is_forum
            ];

            if (!$raw)
            {
                return $return;
            }

            return array_merge($return, [
                'updated_timestamp' => $this->updated_timestamp,
                'updated_client' => $this->updated_client
            ]);
        }

        /**
         * Constructs object from an array representation
         *
         * @param array $array
         * @param bool $raw
         * @return TelegramChatMetadata
         */
        public static function fromArray(array $array, bool $raw=false): TelegramChatMetadata
        {
            $object = new self();

            $object->id = $array['id'];
            $object->type = $array['type'];
            $object->title = $array['title'];
            $object->username = $array['username'];
            $object->first_name = $array['first_name'];
            $object->last_name = $array['last_name'];
            $object->is_forum = $array['is_forum'];

            if(!$raw)
            {
                return $object;
            }

            $object->updated_timestamp = $array['updated_timestamp'];
            $object->updated_client = $array['updated_client'];

            return $object;
        }
    }